--------------------------------------------------------------------------------
-- SatchelFinder.lua
--------------------------------------------------------------------------------

-- this we can use for localisation later --
local SF_INFORMATION_TITLE = "Welcome to Satchel Finder";
local SF_INFORMATION_TEXT = "This addon allows you to join Dungeon Finder only when the |cffffff77Satchel of Exotic Mysteries|r is available for the roles that you selected.\r\rWhen a satchel becomes available the addon will automatically join you to the LFG Queue as the role(s) you have requested.\r\rIt will also remove you from the queue if the rewards disappears.\r\rPlease note that this addon will NOT auto join a group as it is not meant to encourage botting.";

-- variables --
local isRunning = false;
local wasRunning = false;			-- this will be true if we joined a group and then left while scanner was running
local elapsedTime = 0.0;			-- time since last update
local updateInterval = 5.0;			-- request reward info every 5 seconds
local satchelFound = false;			-- satchel reward was found
local isTwilightSatchel = false;	-- satchel reward is for Hour of Twilight Heroic
local isTankReward = false;			-- reward is for tanking 
local isHealerReward = false;		-- reward is for healing
local scanTimer = 0.0;				-- time in seconds that scan has be running
local debugMode = false;

local SEC_TO_MINUTE_FACTOR = 1/60;
local SEC_TO_HOUR_FACTOR = SEC_TO_MINUTE_FACTOR*SEC_TO_MINUTE_FACTOR;
local scanProgress = {".   ", " .  ", "  . ", "   ."};
local scanIndex = 1;
--------------------------------------------------------------------------------
function SFDebugOut(msg)
	if debugMode then
		DEFAULT_CHAT_FRAME:AddMessage(string.format("DebugOut: %s", msg), 1, 1, 0.5);
	end
end

--------------------------------------------------------------------------------
function SatchelFinder_UpdateTimer(elapsed)
	scanTimer = scanTimer + elapsed;
	
	local timer = scanTimer;
	local second = mod(timer, 60);
	
	if second > 1 then
		scanIndex = scanIndex + 1;
		scanTimer = 0;
	end
	
	if scanIndex > 4 then
		scanIndex = 1;
	end
		
	SFMinimapButton.progressText:SetText(scanProgress[scanIndex]);
	
	
end

--------------------------------------------------------------------------------
function SatchelFinder_StartTimer()
	scanTimer = 0.0;
	SatchelFinder_UpdateTimer(0);
end

--------------------------------------------------------------------------------
function SatchelFinder_StopTimer()
	-- SFQueueFrame.timerText:Hide();
end

--------------------------------------------------------------------------------
function SatchelFinder_CreateConfig()
	Settings = { };
	Settings["MinimapPos"] = 45;
	Settings["QueueAsTank"] = false;
	Settings["QueueAsHealer"] = false;
	Settings["QueueForCataclysmHeroics"] = false;
	Settings["QueueForTwilightHeroics"] = false;
	Settings["HideMinimapButton"] = false;
	
	SatchelFinder_LoadConfig()
end

--------------------------------------------------------------------------------
function SatchelFinder_LoadConfig()
	if not Settings then
		SatchelFinder_CreateConfig();
	else
		SFQueueFrameRoleButtonHealer.checkButton:SetChecked(Settings["QueueAsHealer"]);
		SFQueueFrameRoleButtonTank.checkButton:SetChecked(Settings["QueueAsTank"]);
		SFQueueFrameDungeonButtonCataclysm:SetChecked(Settings["QueueForCataclysmHeroics"]);
		SFQueueFrameDungeonButtonHoT:SetChecked(Settings["QueueForTwilightHeroics"]);
		
	end
end

--------------------------------------------------------------------------------
function SatchelFinder_SaveConfig()
	Settings["QueueAsTank"] = SFQueueFrameRoleButtonTank.checkButton:GetChecked();
	Settings["QueueAsHealer"] = SFQueueFrameRoleButtonHealer.checkButton:GetChecked();
	Settings["QueueForCataclysmHeroics"] = SFQueueFrameDungeonButtonCataclysm:GetChecked();
	Settings["QueueForTwilightHeroics"] = SFQueueFrameDungeonButtonHoT:GetChecked();
end

--------------------------------------------------------------------------------
function SatchelFinder_SetStatusMsg(msg)
	DEFAULT_CHAT_FRAME:AddMessage(string.format("SatchelFinder: %s", msg), 0.4, 0.8, 1);
end

--------------------------------------------------------------------------------
function SatchelFinder_StartScan()
	if not isRunning then
		isRunning = true;
		wasRunning = true;
		
		SatchelFinder_StartTimer();
		
		-- Save settings as we have joined
		SatchelFinder_SaveConfig();
		
		-- Lock all dialog controls now --
		LFG_DisableRoleButton(SFQueueFrameRoleButtonHealer);
		LFG_DisableRoleButton(SFQueueFrameRoleButtonTank);
		SFQueueFrameDungeonButtonCataclysm:Disable();
		SFQueueFrameDungeonButtonHoT:Disable();
		SFQueueFrameSearchButton:SetText("Stop");
		SatchelFinder_SetStatusMsg("Starting Scan...");
		
		if not Settings["HideMinimapButton"] then
			SFMinimapButton:Show();
		end
		
		PlaySound("PVPENTERQUEUE");
	end
end

--------------------------------------------------------------------------------
function SatchelFinder_StopScan()
	if isRunning then
		SFMinimapButton:Hide();
		isRunning = false;
		wasRunning = false;
		
		SatchelFinder_StopTimer();
		
		-- Unlock all dialog controls now --
		SatchelFinder_CheckRoles(false);
		SatchelFinder_SetStatusMsg("Stopped Scan...");
		SFQueueFrameSearchButton:SetText("Search");
		
		satchelFound = false;
	end
end

--------------------------------------------------------------------------------
function SatchelFinder_LookForSatchels()
	-- scan for HoT first since we give priority to this if its enabled
	
	-- SFDebugOut("Scanning for Rewards...");
	
	isTankReward = false;
	isHealerReward = false;
	isTwilightSatchel = false;
	satchelFound = false;
	
	if SFQueueFrameDungeonButtonHoT:GetChecked() then
			
		local eligible, forTank, forHealer, forDamage, itemCount, money, xp = GetLFGRoleShortageRewards(434, 1);
				
		if(itemCount ~=0 or money ~=0 and xp ~= 0) then
			isTankReward = forTank;
			isHealerReward = forHealer;
			isTwilightSatchel = true;
			
			local msg = string.format("eligible: %s - forTank: %s - forHealer: %s - itemCount: %s - money: %s",
				tostring(eligible), tostring(forTank), tostring(forTank), tostring(itemCount), tostring(money));
		
			SFDebugOut(msg);
		end
	end
	
	-- scan for cata heroic if enabled
	if SFQueueFrameDungeonButtonCataclysm:GetChecked() then
		
		local eligible, forTank, forHealer, forDamage, itemCount, money, xp = GetLFGRoleShortageRewards(301, 1);
	
		if(itemCount ~=0 or money ~=0 and xp ~= 0) then
			isTankReward = forTank;
			isHealerReward = forHealer;
			isTwilightSatchel = false;
			
			local msg = string.format("eligible: %s - forTank: %s - forHealer: %s - itemCount: %s - money: %s",
				tostring(eligible), tostring(forTank), tostring(forTank), tostring(itemCount), tostring(money));
		
			SFDebugOut(msg);
		end
	end
	
	if isTankReward and SFQueueFrameRoleButtonTank.checkButton:GetChecked() then
		satchelFound = true;
	end
	
	if isHealerReward and SFQueueFrameRoleButtonHealer.checkButton:GetChecked() then
		satchelFound = true;
	end
end

--------------------------------------------------------------------------------
function SatchelFinder_JoinLFG()

	SatchelFinder_SetStatusMsg("Joining Queue...");
	
	local dungeonId = 0;
	
	if isTankReward and SFQueueFrameRoleButtonTank.checkButton:GetChecked() then
		SatchelFinder_SetStatusMsg("Tanking Role Selected");
		SetLFGRoles(true, true, false, false);
	end
	
	if isHealerReward and SFQueueFrameRoleButtonHealer.checkButton:GetChecked() then
		SatchelFinder_SetStatusMsg("Healing Role Selected");
		SetLFGRoles(true, false, true, false);
	end
	
	if isTwilightSatchel and SFQueueFrameDungeonButtonHoT:GetChecked() then
		dungeonId = 434;
		SatchelFinder_SetStatusMsg("Hour of Twilight Heroic Selected");
	end
	
	if not isTwilightSatchel and SFQueueFrameDungeonButtonCataclysm:GetChecked() then
		dungeonId = 301;
		SatchelFinder_SetStatusMsg("Cataclysm Heroic Selected");
	end

	if dungeonId == 0 then
		SatchelFinder_StopScan();
		SatchelFinder_SetStatusMsg("Error - No Valid Dungeon Found!");
		return;
	end
	
	SetLFGDungeon(dungeonId);
	
	JoinLFG();
	
	isRunning = false;
	wasRunning = true;
	
	SatchelFinder_StopScan();
end

--------------------------------------------------------------------------------
function SatchelFinder_RepositionMinimapButton()
	SFMinimapButton:SetPoint("TOPLEFT","Minimap","TOPLEFT",52-(80*cos(Settings.MinimapPos)),(80*sin(Settings.MinimapPos))-52)
end

--------------------------------------------------------------------------------
function SatchelFinder_CheckRoles()
	local canBeTank, canBeHealer, canBeDPS = UnitGetAvailableRoles("player");
	
	if ( canBeTank ) then
		LFG_EnableRoleButton(SFQueueFrameRoleButtonTank);
	else
		LFG_PermanentlyDisableRoleButton(SFQueueFrameRoleButtonTank);
	end
	
	if ( canBeHealer ) then
		LFG_EnableRoleButton(SFQueueFrameRoleButtonHealer);
	else
		LFG_PermanentlyDisableRoleButton(SFQueueFrameRoleButtonHealer);
	end
	
	-- Check Cata Heroic dungeons --
	if IsLFGDungeonJoinable(301) then
		SFQueueFrameDungeonButtonCataclysm:Enable();	
	else
		SFQueueFrameDungeonButtonCataclysm:SetChecked(false);
		SFQueueFrameDungeonButtonCataclysm:Disable();
	end
	
	-- Check Twilight Heroic dungeons --
	if IsLFGDungeonJoinable(434) then
		SFQueueFrameDungeonButtonHoT:Enable();		
	else
		SFQueueFrameDungeonButtonHoT:SetChecked(false);
		SFQueueFrameDungeonButtonHoT:Disable();
	end
end

--------------------------------------------------------------------------------
function SatchelFinder_Initialise()
	SFMinimapButton:Hide();
	
	SatchelFinder_LoadConfig();
	
	SatchelFinder_SetStatusMsg("Satchel Scanner by Klicks (EU-Arathor)");
	SatchelFinder_SetStatusMsg("Type '/sf' or '/satchelfinder' to show");
	
	SFQueueFrame.infoTitle:SetText(SF_INFORMATION_TITLE);
	SFQueueFrame.infoText:SetText(SF_INFORMATION_TEXT);
	
	SatchelFinder_CheckRoles();
	
	SatchelFinder_RepositionMinimapButton();
end

--------------------------------------------------------------------------------
function SatchelFinder_Toggle()
	if SFParentFrame:IsVisible() then
		SFParentFrame:Hide();
	else
		SFParentFrame:Show();
	end
end

function SatchelFinder_OnSlashCommand(msg)
	if msg == "HideMiniButton" then
		Settings["HideMinimapButton"] = true;
		SFMinimapButton:Hide();
	elseif msg == "ShowMiniButton" then
		Settings["HideMinimapButton"] = false;
		if isRunning then
			SFMinimapButton:Show();
		end
	else
		SatchelFinder_Toggle();
	end
end

--------------------------------------------------------------------------------
function SFEventFrame_OnLoad(self)
	self:RegisterEvent("ADDON_LOADED");
	self:RegisterEvent("LFG_UPDATE_RANDOM_INFO");
	
	SLASH_SATCHELFINDER1, SLASH_SATCHELFINDER2 = "/sf", "/satchelfinder";
	SlashCmdList.SATCHELFINDER = SatchelFinder_OnSlashCommand;

end

--------------------------------------------------------------------------------
function SFEventFrame_OnEvent(self, event, ...)
	if event=="ADDON_LOADED" then
		local addonName = ...;
		if addonName == "SatchelFinder" then
			SatchelFinder_Initialise();
		end
	end
	
	if event=="LFG_UPDATE_RANDOM_INFO" then
		if isRunning then
			SatchelFinder_LookForSatchels();
		end
	end
end

--------------------------------------------------------------------------------
function SFEvenFrame_OnUpdate(self, elapsed)
	
	-- Update Dialog --
	if (not SFQueueFrameDungeonButtonHoT:GetChecked() and not SFQueueFrameDungeonButtonCataclysm:GetChecked())
		or (not SFQueueFrameRoleButtonTank.checkButton:GetChecked()and not SFQueueFrameRoleButtonHealer.checkButton:GetChecked())
	then
		SFQueueFrameSearchButton:Disable();
	else
		SFQueueFrameSearchButton:Enable();
	end
	
	if isRunning and satchelFound then
		SatchelFinder_JoinLFG();
	end
	
	-- Update elapsedTime if we are running in scan mode
	if isRunning or isInQueue then
	
		SatchelFinder_UpdateTimer(elapsed);
		
		elapsedTime = elapsedTime + elapsed;
		
		if elapsedTime > updateInterval then
			elapsedTime = 0;
			RequestLFDPlayerLockInfo();
		end
	end
end

--------------------------------------------------------------------------------
function SFParentFrame_OnLoad(self)
	self:RegisterForDrag("LeftButton");
end

--------------------------------------------------------------------------------
function SFParentFrame_OnShow(self)
	PlaySound("igCharacterInfoOpen");
end

--------------------------------------------------------------------------------
function SFParentFrame_OnHide(self)
	PlaySound("igCharacterInfoClose");
end

--------------------------------------------------------------------------------
function SFParentFrame_OnDragStart(self)
	self:StartMoving();
end

--------------------------------------------------------------------------------
function SFParentFrame_OnDragStop(self)
	self:StopMovingOrSizing();
end

--------------------------------------------------------------------------------
function SFQueueFrameSearchButton_OnClick(self)
	if not isRunning then
		SatchelFinder_StartScan();
	else
		SatchelFinder_StopScan();
	end
end

--------------------------------------------------------------------------------
function SFMinimapButton_DraggingFrame_OnUpdate()

	local xpos,ypos = GetCursorPosition();
	local xmin,ymin = Minimap:GetLeft(), Minimap:GetBottom();

	xpos = xmin-xpos/UIParent:GetScale()+70; -- get coordinates as differences from the center of the minimap
	ypos = ypos/UIParent:GetScale()-ymin-70;

	Settings.MinimapPos = math.deg(math.atan2(ypos,xpos)); -- save the degrees we are relative to the minimap center
	SatchelFinder_RepositionMinimapButton(); -- move the button
end

--------------------------------------------------------------------------------
function SFMinimapButton_OnClick()
	SatchelFinder_Toggle();
end